@@ -22,6 +22,8 @@
 #include <unordered_set>
 #include <utility>
 
+#include "metkit/hypercube/HyperCubePayloaded.h"
+
 #include "fdb5/api/helpers/APIIterator.h"
 #include "fdb5/api/helpers/ListElement.h"
 #include "fdb5/database/Key.h"
@@ -30,6 +32,16 @@ namespace fdb5 {
 
 //----------------------------------------------------------------------------------------------------------------------
 
+class ListElementDeduplicator : public metkit::hypercube::Deduplicator<ListElement> {
+public:
+
+    bool toReplace(const ListElement& existing, const ListElement& replacement) const override {
+        return existing.timestamp() < replacement.timestamp();
+    }
+};
+
+//----------------------------------------------------------------------------------------------------------------------
+
 using ListAggregateIterator = APIAggregateIterator<ListElement>;
 
 using ListAsyncIterator = APIAsyncIterator<ListElement>;
